function [GIn, GIp, indN, indP, tbl, pscr, tscr] = matchsum(Nn, Np, max_notes, offdiag)

%Nn is a matrix variable that holds seven pieces of information for each event in the notated piece
%(time, duration,midi channel, note number, velocity, onset(sec), duration (sec)) 
%Np holds this same information for the performed  piece

%the p or n at the end of these variables denotes performed or notated

%max-notes is the maximum number of notes in a chord

% if isempty(Nn) return; end
% if isempty(Np) return; end

% Extract vectors corresponding to onset Times, Pitches,
% and Amplitudes of each note for readability;
    Tn = onset(Nn); Pn = pitch(Nn); An = velocity(Nn);
    Tp = onset(Np); Pp = pitch(Np); Ap = velocity(Nn);


%GP is the vector of chords containing pitches
%GT is the vector of chords containing times
%GI is the vector of chords containing indices

[GPp GTp GIp] = group(Tp, Pp, max_notes, -1); sz= size(GPp); cp = sz(1);
[GPn GTn GIn] = group(Tn, Pn, max_notes, 0); sz= size(GPn); cn = sz(1);

% Here's the Dynamic Programming part
%
[tbl pscr tscr] = table7(GPn, cn, GPp, cp, GTn, GTp, offdiag);

%
% Read the solution from the table
%
indP = [];
indN = [];

i = find(tbl(:,1) == max(tbl(:,1)));		% Find Column Max
i = i(end);                                     % in case more than one
                                                % max, take the last
j = 1;

while i <= cp & j <= cn

   indP = [indP i];
   indN = [indN j];

   % Find CURRENT COLUMN Max (cur) if IOI is short enough to be a chord
   tbl(i+1,j  ) = tbl(i+1,j  ) + tscr(i+1,j);
   cur             = tbl(i+1,j  );
   [curmax nextc] = max(cur);
   if isempty(nextc), nextc = cp+1; else, nextc = i+nextc(1); end;

   % Find NEXT ROW Max
   tbl(i+1,j+1:cn) = tbl(i+1,j+1:cn) - tscr(i+1,j+1:cn);
   row    = tbl(i+1,j+1:cn);
   [rowmax nextj] = max(row);
   if isempty(nextj), nextj = cn+1; else, nextj = j+nextj(1); end;

   % Find NEXT COLUMN Max (note: mistake in the paper)
   col    = tbl(i+1:cp,j+1) - tscr(i+1:cp,j+1);
   if ~isempty(col), col(1) = col(1) + tscr(i+1,j+1); end;
   
   [colmax nexti] = max(col);

   if isempty(nexti)
      nexti = cp+1; 
   else
      nexti = i+nexti(1); 
   end;

   if ~isempty(colmax)
      if colmax >= curmax & colmax >= rowmax 
	 tbl(i+1:nexti,j+1) = tbl(i+1:nexti,j+1) - tscr(i+1:nexti,j+1);
	 tbl(i+1,j+1) = tbl(i+1,j+1) + tscr(i+1,j+1);
      end

      if colmax < curmax
	 colmax = curmax;
	 nexti = nextc;
	 jplus = j;
      else
	 jplus = j+1;
      end
   end
   
   if isempty(colmax); colmax = -1; end;
   if isempty(rowmax); rowmax = -1; end;

   if ~isempty(colmax) & ~isempty(rowmax)
      if colmax >= rowmax
	 i = nexti;
	 j = jplus;
      else
	 i = i+1;
	 j = nextj;
      end
   end

end

% Sort the indices in GIp and GIn into their original order
% which may have been screwed up above

for i = 1:size(GIp,1)
   notei = find(GIp(i,:));
   GIp(i,notei) = sort(GIp(i,notei));
end
for i = 1:size(GIn,1)
   notei = find(GIn(i,:));
   GIn(i,notei) = sort(GIn(i,notei));
end
