function [tbl, pscr, tscr] = table(GPn, nn, GPp, np, GTn, GTp, offdiag)

tbl  = sparse(zeros(np+1, nn+1));
pscr = sparse(zeros(np+1, nn+1));
tscr = sparse(zeros(np+1, nn+1));

% Equate mean tempi
%tempoRatio finds the ratio of the performed tempo to the notated tempo
%the time matrix of the notated is then multiplied by the ratio to change
%the average tempo of the notated to match that of the performed - they
%now begin and end at the same time
tempoRatio = GTp(size(GTp,1),1)/GTn(size(GTn,1),1);
GTn = GTn * tempoRatio;

% Range limit for efficiency
slope = nn/np;
range = offdiag;		% this should be a
                                % parameter (now it is: 11/11/09 EWL)

for i = np:-1:1
    nn_upper = min(nn, round(slope*i+range));
    nn_lower = max( 1, round(slope*i-range));
    for j = nn_upper:-1:nn_lower

        %  pitch score
        pscr(i,j) = scorep(GPn(j,:), GPp(i,:));

        %  interval time score
        tscr(i,j) = scoret(GTp, GTn, i, j, tempoRatio);


        curmax  = tbl(i+1, j) + tscr(i+1, j);

        col = tbl(i+1:np,j+1);
        if isempty(col)
            col  = 0;
        else
            col(1)  = col(1) - tscr(i+1,j+1);
        end
        colmax  = max(col);

        row = tbl(i+1,j+1:nn);
        if isempty(row)
            row = 0;
        else
            row = row - tscr(i+1,j+1:nn);
        end
        rowmax  = max(row);

        tbl(i,j) = pscr(i,j) + max([curmax, colmax, rowmax]);

    end
end

